pub enum PseudoOp {
CallOther,
ConstantPoolRef,
New,
}Expand description
Operations which represent black-box placeholders for some sequence of changes to the machine state.
Variants
CallOther
A call that cannot be semantically represented in p-code. For example, a syscall.
ConstantPoolRef
Returns specific run-time dependent values from the constant pool. Used by object-oriented instruction sets and other managed code environments.
New
Allocates memory for an object or set of objects.