InstructionLoader

Trait InstructionLoader 

pub trait InstructionLoader {
    // Required method
    fn load_instruction_bytes(
        &self,
        source: &VarnodeData,
    ) -> Result<Vec<u8>, String>;
}
Interface for loading instruction bytes to be disassembled.

Required Methods

fn load_instruction_bytes( &self, source: &VarnodeData, ) -> Result<Vec<u8>, String>

Load instruction bytes from the requested source. If not all of the requested bytes are available, then the initial sequence of bytes which are available should be returned. For example, if the caller requests 30 bytes but only the first 10 are available, only those 10 should be returned.

Implementors